c
c     Copyright 1996, 2001 Systems Applications International
c       The REMSAD modeling system software is the property of SAI
c       and is protected by registered copyrights. Distribution of
c       the software outside of your organization is not permitted.
c       Preparation and distribution of any derivative software using
c       any portions of the REMSAD code, without the express permission
c       of SAI, is also not permitted. By accepting the REMSAD code
c       provided in this release, you are also accepting the
c       limitations, terms, and provisions of use as noted above.
      subroutine beh084(stkht,stkd,vexit,texit0,zs,w,temp,press,nz,
     &                  heff)
c
c-----ATDM v1.00
c
c     Calculates plume rise and effective stack height
c
c-----95/06/02:
c     -from UAM-V Revision 1.3  94/07/29  15:53:01  tcm
c     -modifications to neutral momentum rise following similar
c      changes to UAM PTSRCE preprocessor
c     -replaced mxz with nz in 2 statements
c
c     Calculate plume rise using:
c      1) Neutral-Unstable plume rise equations of Briggs (1984)
c      2) Stable plume rise equations of Briggs (1975)
c     This routine is based on a algorithm from TUPOS (Turner and 
c     Catalano, 1986, US EPA).  Plume rise is adjusted due to stack tip
c     downwash (Bjorklund and Bowers, 1982).  Stable algorithm requires
c     potential temperature lapse rate, which is obtained from temp and 
c     pres arrays, and is considered valid between layer midpoints.
c     Temperature and winds are considered layer average, and are considered
c     valid at layer midpoints.
c
c     Internal Variables:
c       delh       final plume rise (m)
c       dwnwsh     stack tip downwash factor (0 to 1)
c       f0         bouancy flux out of stack (m4/s3)
c       fb         bouancy flux at bottom of a layer
c       fr         residual bouyancy flux at top of layer
c       flux       generic flux (one of the fluxes above)
c       froude     stack froude number
c       nsteps     number of iterations to final plume rise
c       tmp        ambient temperature (K)
c       dtdz       ambient potential temperature gradient (K/m)
c       wind       ambient windspeed (m/s)
c       zb         height of layer bottom (m)
c       zt         height of layer top (m)
c       kgrad      layer index for gradient calculations
c       kstk       layer index for stack
c
c     Input arguments:
c               stkht:          physical stack height (m)
c               stkd:           stack diameter (m)
c               vexit:          exit velocity (m/s)
c               texit0:         exit temperature (K)
c               zs:             layer interface height array (m)
c               w:              layer wind array (m/s)
c               temp:           layer temperature array (K)
c               press:          layer pressure (mb)
c               nz:             number of layers
c     Output arguments:
c               heff:           final effective stack height (m)
c     Files written:
c               6:              standard output file unit (*)
c     Called by:
c               update
c
      include 'parm.cmd'
      real zs(nz), w(nz), press(nz), temp(nz)
      real dthdz(mxz)
c
      nsteps = 1
      delh = 0.
      g = 9.8
      gamma = 0.286
c
c-----Initialize potential temperature array 
c
      do 5 k = 1,nz-1
        if (k.eq.1) dz = zs(k+1)/2.
        if (k.gt.1) dz = (zs(k+1) - zs(k-1))/2.
        dthdz(k) = (1000.**gamma/dz)
     &              *(temp(k+1)/press(k+1)**gamma
     &              - temp(k)/press(k)**gamma)
    5 continue
c
c-----Determine layer index in which stack resides and initialize some
c     variables
c
      do 10 k = 1,nz
        if (stkht.lt.zs(k)) then
          kstk = k
          goto 20
        endif
   10 continue
   20 continue
c     write(*,*)'BEH084:MAde it to line 20'
c
      wind = amax1(w(kstk),0.1)
      tmp = temp(kstk)
      zt = zs(kstk) - stkht
      zb = 0.
c     write(*,*)'BEOH084: kstk,w,temp,zt == ',kstk,wind,tmp,zt
c
c-----Use stack temperature of at least .1 deg higher than ambient
c     (tcm, 10/21/91)
c
      texit = texit0
      if (texit0 .lt. tmp+0.1) texit = tmp+0.1
c
c-----Calculate Froude number and stack tip downwash correction factor
c
      froude = vexit**2./(g*stkd*(texit - tmp)/tmp)
      dwnwsh = 1.
c     write(*,*)'BEOH084: texit,froude == ',texit,froude
      if (froude.ge.3.) then
        if (wind.ge.vexit) then
          dwnwsh = 0.
          goto 1000
        elseif ((wind.gt.vexit/1.5) .and. (wind.lt.vexit)) then
          dwnwsh = (3.*vexit - 3.*wind)/vexit
        endif
      endif
c
c-----Find Initial buoyancy flux and neutral-unstable momentum rise
c
      delhum = 3.*stkd*vexit/wind
      if (delhum.gt.zt) then
        do 50 kend = kstk+1,nz 
          fcum = zt
          wcum = fcum*w(kstk)
          do 40 k = kstk+1,kend
            ff = amin1(delhum+stkht,zs(k)) - zs(k-1)
            fcum = fcum + ff
            wcum = wcum + ff*w(k)
  40      continue
          wwav = amax1(wcum/fcum,0.1)
          delhum = 3.*stkd*vexit/wwav
          if (delhum.lt.zs(kend)-stkht) goto 55
  50    continue
      endif
  55  f0 = g*vexit*stkd**2.*(texit - tmp)/(4.*texit)
      flux = f0
c     write(*,*)
c     write(*,*) '********************************************'
c     write(*,*) 'froude,dwnwsh,delhum ',froude,dwnwsh,delhum
c
c-----Loop by layer to find plume rise, initialize some variables
c
  100 continue
      kgrad = kstk
      if (nsteps.eq.1) then
        if (kstk.gt.1) then
          if ((stkht+delh).lt.(zs(kstk)+zs(kstk-1))/2.) kgrad = kstk-1
        endif
        if (kstk.eq.nz) kgrad = kstk-1
      else
        kgrad = kstk-1
        flux = fr
        wind = amax1(w(kstk),0.1)
        tmp = temp(kstk)
        zt = zs(kstk) - stkht
        zb = zs(kstk-1) - stkht
      endif
      dtdz = dthdz(kgrad)
c     write(*,*)'flux,wind,temp,zt,zb,kgrad,k,dtdz '
c     write(*,*)flux,wind,tmp,zt,zb,kgrad,kstk,dtdz
      if (dtdz.lt.0.001) goto 200 
      if (dtdz.ge.0.001) goto 300
c
c---------------------------------------------------------------------
c-----Neutral-unstable case
c
  200 continue
c
c-----Neutral-unstable bouyancy rise
c
c     write(*,*) '*****Unstable case*****'
      delhub1 = 30.*(flux/wind)**0.6 + zb
      delhub2 = 24.*(flux/wind**3.)**0.6
     &          *(stkht + 200.*(flux/wind**3.))**0.4 + zb
      delhub = amin1(delhub1,delhub2)
      if (delhub.eq.delhub1) iub = 1
      if (delhub.eq.delhub2) iub = 2
c     write(*,*)'delhub',delhub
c
c-----Neutral-unstable rise is maximum of momentum and buoyancy rise
c
      if ((nsteps.eq.1).and.(delhum.gt.delhub)) then
c       write(*,*)'using delhum'
        delh = delhum
        goto 1000
      endif
c     write(*,*)'using delhub'
      delh = delhub
c     if (delh.lt.zt) goto 1000
      if (delh.le.zt) goto 1000    ! avoid divide by 0, tcm 7/12/01
c
c-----Calculate residual buoyancy if plume rise is beyond layer containing
c     stack
c
c     write(*,*)'calculating fr'
      if (iub.eq.1) fr = wind*((delh - zt)/30.)**(5./3.)
      if (iub.eq.2) fr = 5.5e-3*(delh - zt)*wind**3./
     &                   (1. + stkht/(delh-zt))**(2./3.)
      if(fr.le.0.) goto 1000
      nsteps = nsteps + 1
      kstk = kstk + 1
      if(kstk.gt.nz) goto 1000
c     write(*,*)'fr nsteps ',fr,nsteps
      goto 100
c
c----------------------------------------------------------------------
c-----Stable case
c
  300 continue
c
c     write(*,*)'*****Stable case****'
c-----Determine stable momentum rise (use minimum of stable and neutral-
c     unstable momentum rise as momentum rise in stable conditions)
c
c     write(*,*) 'vexit,stkd,texit,wind,tmp,dtdz: '
c     write(*,*) vexit,stkd,texit,wind,tmp,dtdz
      delhsm = 0.646*(vexit**2.*stkd**2./(texit*wind))**(1./3.)
     &         *tmp**0.5/(dtdz**(1./6.))
      delhsm = amin1(delhsm,delhum)
c     write(*,*)'delhsm',delhsm
c
c-----Stable buoyancy rise
c
      delhsb1 = (1.8*flux*tmp/(wind*dtdz) + zb**3.)**(1./3.)
      delhsb2 = (4.1*flux*tmp/(f0**(1./3.)*dtdz)
     &          + zb**(8./3.))**(3./8.)
      delhsb = amin1(delhsb1,delhsb2)
      if (delhsb.eq.delhsb1) isb = 1
      if (delhsb.eq.delhsb2) isb = 2
c     write(*,*)'delhsb',delhsb
c
c-----Stable plume rise is maximum between stable momentum rise and 2/3
c     stable bouyancy rise of top of plume
c
c     if ((nsteps.eq.1).and.(delhsm.gt.(2.*delhsb/3.))) then
      if ((nsteps.eq.1).and.(delhsm.gt.delhsb)) then
        delh = delhsm
c       write(*,*)'using delhsm'
        goto 1000
      endif
c     write(*,*)'using delhsb'
      delh = delhsb
      if (delh.lt.zt) then
c       delh = 2.*delh/3.
        goto 1000
      endif
c
c-----Calculate residual buoyancy if top of plume is beyond layer containing
c     stack
c
c     write(*,*)'calculating fr'
      if (nsteps.eq.1) then
        fb = f0
      else
        fb = fr
      endif
      if (isb.eq.1) fr = fb - (0.56*dtdz*wind/tmp)
     &                   *(zt**3. - zb**3.) 
      if (isb.eq.2) fr = fb - (0.24*dtdz*f0**(1./3.)/tmp)
     &                   *(zt**(8./3.) - zb**(8./3.))
      if(fr.le.0.) then
c       delh=2.*delh/3.
        goto 1000
      endif
      nsteps = nsteps + 1
      kstk = kstk + 1
      if(kstk.gt.nz) then
c       delh=2.*delh/3.
        goto 1000
      endif
c     write(*,*)'fr,nsteps',fr,nsteps
      goto 100
c
c-----Final plume rise
c
 1000 heff = stkht + dwnwsh*delh 
c     write(*,*)'Exiting beh084'
      call flush(6)
      return
      end
