c
c     Copyright 1996, 2001 Systems Applications International
c       The REMSAD modeling system software is the property of SAI
c       and is protected by registered copyrights. Distribution of
c       the software outside of your organization is not permitted.
c       Preparation and distribution of any derivative software using
c       any portions of the REMSAD code, without the express permission
c       of SAI, is also not permitted. By accepting the REMSAD code
c       provided in this release, you are also accepting the
c       limitations, terms, and provisions of use as noted above.
      subroutine rchem2f(time,idate,dtf,idebug,nxf,nyf,nzf,nfspec,
     &                  ifine,rainf,htf,presf,temprf,clh2of,rnh2of,ch2of,wratf,
     &                  fconvf,fstrtf,kcldbf,kcldtf,kfrzf,xmdf,pstarf,
     &                  ozonef,h2o2f,ctlonf,ctlatf,cloudf,avterrf,cncf,
     &                  wpositf,ohconf,no3conf,ho2conf,aso2conf,ffnonpr,
     &                  qfarea)
c
c-----ATDM v1.00
c
c     Chemistry driver for the fine grids (for Micro CB4 mechanism only)
c
c-----95/06/19:
c     C Emery SAI
c-----95/10/05:
c     -revised calculation of PM10 for toxic partitioning to be in units
c      of ug/m3 rather than ppm
c     -reworked aqueous chemistry code, cleaned up some errors
c-----95/10/06:
c     -removed zmsl array; passing PHOT zcell and avterr
c-----95/10/18:
c     -restructured for more efficient handling of lwet and rain
c-----95/10/26:
c     -revised calculation of Hg0 -> HgII aqueous reaction
c-----95/12/08: gem
c     -fixed index on bdnl from k to l
c-----95/12/11: gem
c     -changed pm10=0. to pm10=4.  (bckgrnd pm10)
c-----95/12/13
c     - added check against lower bndl after call to nitrat
c-----95/12/22:
c     -added branch around convmix for PM runs, calls wetdep from here
c      (quick fix to avoid problems in convmix with strong convective
c       columns with low cloud base)
c-----96/07/29:
c     -moved ierr check inside if-block (see comment from 95/12/22)
c-----96/09/16:dfg
c     -added call to tdrate, photol, step4 for solve micro CB4 chemical
c      mechanism.
c     -removed call to partchm for pm chemistry.
c     -moved calculation of cloud mixing/wet dep to the end of chemistry
c      loop.
c     -added one concentration array con(mxspec).
c     -added ozonef(i,j,k)=cncf(i,j,ko3,k)
c     -added include file comchem2.cmd
c-----97/03/24:DFG
c     -passed k to nitrat subroutine.
c-----97/06/25:DFG
c     -passed pressure (i.e. pecll(k)) to nitrat subroutine.
c-----97/12/20:DFG
c     -set ddtt=dtf in responding to unit of per min for k's.
c-----99/4/14: DFG
c     -added fine radical array for new chemistry solver to use.
c     -added radical concentration (RCON) to passing input to step4.
c-----99/4/22: DFG
c     -added loops for saving OH separately when necessary.
c
c     Rebuilt to contain the cloud sub-model; for those columns with
c     precipitation ( > 0.01 in/hr), the vertical concentration profile
c     is split between an in-cloud profile and non-cloud profile.  Cloud
c     mixing and wet deposition act on the in-cloud profile over multiple
c     sub-steps, after which the two profiles are recombined.  PM and/or
c     toxic chemistry is then performed on the combined profile.
c
c     Input arguments:
c               time:           model time
c               idate:          model date
c               dtf:            fine grid time step (min)
c               idebug:         diagnostic debug flag
c               nxf:            number of fine grid cell in x-direction
c               nyf:            number of fine grid cell in y-direction
c               nzf:            number of layers
c               nfspec:         number of modeled species
c               ifine:          fine grid index
c               rainf:          rainfall rate (in/hr)
c               htf:            array of layer interface heights (m)
c               presf:          array of pressure (mb)
c               temprf:         array of temperature (K)
c               ch2of:          array of water vapor concentration (ppm)
c               wratf:          wet scavenging rate for sulfate (1/s)
c               fconvf:         fraction of precipitating convective coverage
c               fstrtf:         fraction of precipitating stratus coverage
c               kcldbf:         layer index of cloud bottom
c               kcldtf:         layer index of cloud top
c               kfrzf:          layer index of freezing level
c               xmdf:           convective mixing coefficient (1/s)
c               pstarf:         array of fine grid P* (mb)
c               ctlonf:         gridded longitude (deg)
c               ctlatf:         gridded latitude (deg)
c               cloudf:         gridded cloud cover (decimal fraction)
c               avterrf:        gridded mean terrain elevation (m MSL)
c               cncf:           array of fine grid concentrations (ppm)
c     Output arguments:
c               cncf:           array of fine grid concentrations (ppm)
c               wpositf:        array of fine grid deposited mass (g/km2) 
c     Files written:
c               6:              standard output file unit (*)
c     Routines called:
c               jtocal    nitrat    toxpart   convmix   strtmix  solar
c               photol    step4     toxchm
c     Called by:
c               atdm
c
      include 'parm.cmd'
      include 'parchm.cmd'
      include 'parfin.cmd'
      include 'deffin.cmd'
      include 'comuam.cmd'
      include 'comchem.cmd'
      include 'comchem2.cmd'
      include 'ohtbl.cmd'
      include 'aerosl.cmd'
      include 'balanc.cmd'
      include 'ctlflg.cmd'
      include 'latlon.cmd'
      include 'com3.cmd'
      include 'kweight.cmd'
c
      dimension aradconf(mxrads,mxfz)
      dimension ixff0(maxfin), ixff1(maxfin),
     &          jyff0(maxfin), jyff1(maxfin)
      dimension cncf(nxf,nyf,nfspec,nzf), htf(nxf,nyf,nzf),
     &          presf(nxf,nyf,nzf), temprf(nxf,nyf,nzf),
     &          clh2of(nxf,nyf,nzf), rnh2of(nxf,nyf,nzf),
     &          ch2of(nxf,nyf,nzf), rainf(nxf,nyf), wratf(nxf,nyf,nzf),
     &          ctlonf(nxf,nyf), ctlatf(nxf,nyf), pstarf(nxf,nyf),
     &          cloudf(nxf,nyf,nzf), ozonef(nxf,nyf,nzf),
     &          wpositf(nxf,nyf,nfspec), ffnonpr(nxf,nyf,nzf)
      dimension fconvf(nxf,nyf),fstrtf(nxf,nyf),kcldbf(nxf,nyf),
     &          kcldtf(nxf,nyf),kfrzf(nxf,nyf),xmdf(nxf,nyf),
     &          avterrf(nxf,nyf)
      dimension cincld(mxfz,mxspec), cnocld(mxfz,mxspec)
      dimension acincld(mxfz), acnocld(mxfz)
      dimension zcell(mxfz),tcell(mxfz),pcell(mxfz),qcell(mxfz),
     &          rhcell(mxfz),ccell(mxfz)
      dimension wi(mxfz), delhg0(mxfz), delgso4(mxfz)
      dimension pm(7), tox(5)
      dimension con(mxspec),rcon(mxrads)
      dimension ohconf(nxf,nyf,nzf),no3conf(nxf,nyf,nzf)
      dimension h2o2f(nxf,nyf,nzf),ho2conf(nxf,nyf,nzf)
      dimension aso2conf(nxf,nyf,nzf)
      dimension flwch2or(mxfx,mxfy,mxfz)
      dimension fclwch2or(mxfx,mxfy,mxfz)
      dimension qfarea(nxf,nyf,nfspec)
c
      dimension flqdh2o(mxz)
      real*4 hf(3)
      real svg(4), svp(4), mwv(4)
      logical lprnt,lpro3
      real nh3, nh4n, nh4s, lv, lwc, mfac, no3conf
      common /idcell/idex,jdex,kdex,tmchm,ifg
      save ndone
      data eps /0.622/, lv /2.5e6/, e0 /6.11/, rv /461./
      data lwc /1./, ekho30 /1.13e-2/, tdepo3 /-2300./, rkhg0 /2.31e+8/
      data pm /7*0./, tox /5*0./
      data pi /3.1415927/
      data ndone /0/
      data nsv /4/
c
      write(*,3001) dtf
 3001 format(' enter rchem2f  ',f10.2)
      cpusoa = 0.
c
      do i = 1, nxf
         do j = 1, nyf
            do k = 1, nzf
               do is = 1, nfspec
                  if (cncf(i,j,is,k) .lt. 0.0) then
                     write (*,*) 'rchem2f',i,j,k,is,cncf(i,j,is,k)
                     stop
                  end if
               end do
            end do
         end do
      end do
c   Initilizing the array con for step4 use.
      do l=1,mxspec
        con(l)=0.
      end do
c
c----Bit five of idebug on (i.e., the 16 location) will turn
c    on debug writes in rchemf.
c
      idiag = and(idebug,16)
      lprnt = .false.
      if (idiag.ne.0) lprnt = .true.
c
c----Set up some parameters relating the grids
c
      do 5 if1 = 1,numfin
        if (ifgptr(if1) .ne. ifine) go to 5
        ixff0(if1) = (ixfb(if1) - ixfb(ifine))*nhf(ifine) + 2
        ixff1(if1) = (ixfe(if1) - ixfb(ifine) + 1)*nhf(ifine) + 1
        jyff0(if1) = (jyfb(if1) - jyfb(ifine))*nhf(ifine) + 2
        jyff1(if1) = (jyfe(if1) - jyfb(ifine) + 1)*nhf(ifine) + 1
        if (ixff0(if1).le.2 .or. ixff1(if1).ge.nxf .or.
     &      jyff0(if1).le.2 .or. jyff1(if1).ge.nyf) then
          write(*,*) ' erroneous grid parameters in rchemf'
          write(*,*) ' ixff0, ixff1, jyff0, jyff1: ',ixff0,ixff1,
     &                 jyff0, jyff1
          write(*,*) 'if1, ifine: ',if1,ifine
          stop
        endif
   5  continue
c
c-----Extract date
c
      if (idate.lt.100000) then
        call jtocal(idate,icdate)
        nyr = icdate/10000
        nmth = icdate/100 - nyr*100
        nday = icdate - nmth*100 - nyr*10000
      else
        nyr = idate/10000
        nmth = idate/100 - nyr*100
        nday = idate - nmth*100 - nyr*10000
      endif
      nyr = nyr + 1900
c
c for given time interval, calculate the liquid water content for entire domain
c
      do i = 1, nxf
         do j = 1, nyf
            do k = 1, nzf
               if (clh2of(i,j,k).eq.0.0) then
                  if (rnh2of(i,j,k).eq.0.0) then
                     flwch2or(i,j,k) = 0.0
                     fclwch2or(i,j,k) = 0.0
                     go to 25
                  end if
               end if
               call falwc(clh2of(i,j,k),rnh2of(i,j,k),presf(i,j,k),
     *                    temprf(i,j,k),nxf,nyf,nzf,flwch2or(i,j,k),
     *                    fclwch2or(i,j,k))
 25            continue
            end do
         end do
      end do
c     
c*** 
c   Convert time step to hour for step4 to use.
c***
c***DFG
c     ddtt = dtf/60.0
c
      ddtt = dtf
c
c-----Loop on grids
c
      do 100 j = 2,nyf-1
        do 99 i = 2,nxf-1
c
           do ijk = 1, nlayer
              flqdh2o(ijk)= fclwch2or(i,j,ijk)
           end do
c
c-----Skip cells covered by another finer grid
c
          do 10 if1 = 1,numfin
            if (ifgptr(if1).eq.ifine) then
              if (i.ge.ixff0(if1) .and. i.le.ixff1(if1)
     &            .and. j.ge.jyff0(if1) .and. j.le.jyff1(if1))
     &            go to 99
            endif
   10     continue
c
c-----Load some ambient column profiles
c
          do 20 k = 1,nzf
            zcell(k) = htf(i,j,k)
            tcell(k) = temprf(i,j,k)
            pcell(k) = presf(i,j,k)
            qcell(k) = ch2of(i,j,k)
            ccell(k) = cloudf(i,j,k)
            h2o = 1.e6*qcell(k)*28.8/18.
            ev = qcell(k)*pcell(k)/(qcell(k) + eps)
            es = e0*exp((lv/rv)*(1./273. - 1./tcell(k)))
            rhcell(k) = 100.*ev/es
  20      continue
c
c-----Determine nitrate equilibrium if this is a PM chemistry run, or if
c     its a Toxic chemistry run with nitrates
c
c
          do 30 k = 1,nzf
            if (lpmchm .or. (ltxchm .and. khno3.ne.0)) then
c
c               aso4 and gso4 should not be modified by remeql3
c               so calculation of fractions and redistribution
c               of sulfate has been removed.
c               since prior versions of REMSAD have assumed
c               that all pariculate nitrate is nh4no3,
c               we will assume that here also.  This means
c               that we do not need to calculate the fraction
c               of nh4 as nitrate (frnh4n) before remeql3.
c               The assumption that this fraction would be
c               the same after remeql3 is not correct anyway.
c               (tcm, 10/18/00)
                  tso4  = cncf(i,j,kaso4,k) + cncf(i,j,kgso4,k)
                  thno3 = cncf(i,j,khno3,k)
                  tnh3o = cncf(i,j,knh3,k)
                  pno3  = cncf(i,j,kpno3,k)
                  panh4 = cncf(i,j,knh4n,k) + cncf(i,j,knh4s,k)
Ctcm              frnh4n= cncf(i,j,knh4n,k)/panh4
Ctcm              paso4 = cncf(i,j,kaso4,k) + cncf(i,j,kgso4,k)
Ctcm              fraso4= cncf(i,j,kaso4,k)/paso4
                  call remeql3(tso4,thno3,tnh3o,rhcell(k),tcell(k),
     &                 pcell(k),pno3,panh4,paso4,i,j,k)
c
c     check against lower bound for these species
c
                  cncf(i,j,khno3,k) = amax1(thno3,bdnl(khno3))
                  cncf(i,j,knh3,k)  = amax1(tnh3o,bdnl(knh3))
Ctcm              cncf(i,j,kaso4,k) = amax1(paso4*fraso4,bdnl(kaso4))
Ctcm              cncf(i,j,kgso4,k) = amax1(paso4*(1-fraso4),bdnl(kgso4))
                  cncf(i,j,kpno3,k) = amax1(pno3,bdnl(kpno3))
Ctcm              cncf(i,j,knh4n,k) = amax1(panh4*frnh4n,bdnl(knh4n))
Ctcm              cncf(i,j,knh4s,k) = amax1(panh4*(1-frnh4n),bdnl(knh4s))
                  cncf(i,j,knh4n,k) = amax1(pno3,bdnl(knh4n))
                  cncf(i,j,knh4s,k) = amax1(panh4-pno3,bdnl(knh4s))

            endif
c
c-----Partition certain toxic species between vapor and particle phase
c
            if (ltxchm) then
              pm10 = 4.
              mfac = cpfac*(1013.25/pcell(k))*(tcell(k)/273.)
              do 35 l = ngas+1,nfspec
                if (l.eq.kpom1p .or. l.eq.kpom2p .or. l.eq.kdioxp .or.
     &              l.eq.katrazp) goto 35
                lsp = jind(l)
                pm10 = pm10 + molwt(lsp)*cncf(i,j,l,k)/mfac
  35          continue
              if (kpom1.ne.0)
     &          call toxpart(pm10,tcell(k),cncf(i,j,kpom1,k),
     &                       cncf(i,j,kpom1p,k),1)
              if (kpom2.ne.0)
     &          call toxpart(pm10,tcell(k),cncf(i,j,kpom2,k),
     &                       cncf(i,j,kpom2p,k),2)
              if (kdiox.ne.0)
     &          call toxpart(pm10,tcell(k),cncf(i,j,kdiox,k),
     &                       cncf(i,j,kdioxp,k),3)
              if (katraz.ne.0)
     &          call toxpart(pm10,tcell(k),cncf(i,j,katraz,k),
     &                       cncf(i,j,katrazp,k),4)
            endif
   30     continue
c
c-----Calculate solar zenith angle
c
          call solar(ctlatf(i,j),-ctlonf(i,j),tz,nyr,nmth,nday,time,
     &               zcoef)
          zenith = (180./pi)*acos(zcoef)
c
c-----Do chemistry in each layer
c
          do 50 k = 1,nzf
            rh=rhcell(k)
c
c*** Call tdrate to get rate constants and do chemistry when
c    mechanism is specified.
c
          h2o = 1.e6*qcell(k)*28.8/18.
          bh2o = 1.e6*qcell(k)*28.8/18.
          call tdrate(tcell(k),pcell(k))
          call rate_coeff(tcell(k),pcell(k),bh2o,pcell(1))
c
c-----Obtain current photolysis rates from lookup table
c
          call photol(rk,rkn,k,nzf,zenith,zcell,
     &             avterrf(i,j),ccell,ldark)
c
c          write (*,*) 'after tdrate in rchem2f'
c          write (*,*) i,j,k
c          write (*,*) h2o,tcell(k),pcell(k)
c          do irxn = 1, 64
c             write (*,*) irxn, rk(irxn)
c          end do
c          stop
c
c    Load layer-average concentration into con array.
c
            do 31 l=1,nors
              con(l) = cncf(i,j,l,k) 
 31         continue
c
c  load radical concentrations
c
            llx = i + nxf*(j-1) + nxf*nyf*(k-1)
            do ll=1,mxrads
              rcon(ll)=radconf(ll,ifine,llx)
            end do
c
c   Save the previous concentration of SULF.
c
          rsulf1=cncf(i,j,ksulf,k)
c
          aromf = 0.
          if (katol .gt. 0) aromf = qfarea(i,j,katol)
c
          itest = itest + 1
          idex = i
          jdex = j
          kdex = k
          testtime = time
          testidate = idate
c          write (*,*) i,j,k,tcell(k),pcell(k),h2o
c          write (*,*) (rk(ii),ii=1,63)
          if (con(2) .gt. .9) then
            write(*,*) 'Before step4, con(2) = ', con(2)
            write(*,*) 'idex,jdex,kdex: '
            write(*,*) idex,jdex,kdex
          endif
          ifg = ifine
c         lprnt = .false.
c         if (i.eq.18 .and. j.eq.14 .and. k.eq.1 .and. ifg.eq.3) then
c         if (i.eq.55 .and. j.eq.7 .and. ifg.eq.2) then
c           write(*,*) 'Before step4f, time, i,j,k,tcell,ifg: '
c           write(*,*) time, i,j,k,tcell(k),ifg
c           write(*,*) 'h2o = ', h2o
c           write(*,*) 'Con: ', con
c           write(*,*) 'Rcon: ', rcon
c           lprnt = .true.
c         endif
          call step4(ddtt,con,tcell(k),lprnt,rcon,time,idate)
 1002     format (100(1x,e10.4))
c         lprnt = .false.
c         if (i.eq.18 .and. j.eq.14 .and. k.eq.1 .and. ifg.eq.3) then
c         if (i.eq.55 .and. j.eq.7 .and. ifg.eq.2) then
c           write(*,*) 'After step4f, time, i,j,k,tcell,ifg: '
c           write(*,*) time, i,j,k,tcell(k),ifg
c           write(*,*) 'Con: ', con
c           write(*,*) 'Rcon: ', rcon
c         endif
c
c   Load back conc array.
          do 32 l=1,nors
            cncf(i,j,l,k) = con(l)
 32       continue
          do ll=1,mxrads
             radconf(ll,ifine,llx)=rcon(ll)
             aradconf(ll,k)=rcon(ll)
          end do
c
c   save oh concentration separately if necessary.  dfg,4/14/99
c
          if (lmoh) ohconf(i,j,k)=rcon(koh)
          if (lmoh) ho2conf(i,j,k)=rcon(kho2)
          if (lmoh) no3conf(i,j,k)=rcon(kno3)
c
c   Save the new concentration of SULF.
c
          rsulf2=cncf(i,j,ksulf,k)
c
          ozonef(i,j,k)=cncf(i,j,ko3,k)
          h2o2f(i,j,k)=cncf(i,j,kh2o2,k)
c
c-----Load SO2, GSO4, ASO4 concentrations to PM array and 
c     Calculate sulfate concentrations
c
          pm(1) = cncf(i,j,kso2,k)
          pm(2) = cncf(i,j,kgso4,k)
          pm(3) = cncf(i,j,kaso4,k)
          pm(4) = cncf(i,j,kh2o2,k)
          pm(5) = cncf(i,j,ko3,k)
          rsulf = rsulf2 - rsulf1
c
c-----calculate eff. henry's law constants for h2o2 and so2
c
          hplus = 10.**(-4.5)
          hplus2 = hplus*hplus
          do nh = 1,3
            ttmp1 = kt(nh)*aqrk1(nh)/hplus
            ttmp2 = kt(nh)*aqrk1(nh)*aqrk2(nh)/hplus2
            hf(nh) = kt(nh) + ttmp1 + ttmp2
          enddo
c
c-----Call sulfate chemistry routine.
c
          fnonpr = ffnonpr(i,j,k) 
          vmr = clh2of(i,j,k)*1.293e-3*pcell(k)/1013.*273./tcell(k)
          call sulfate(pm,so2av,rsulf,vmr,tcell(k),pcell(k),
     &                 hf,dtf,lprnt,ccell(k),fnonpr)
c
c-----Load SO2, GSO4, ASO4 concentrations back to CONC array and 
c
            cncf(i,j,kso2,k) = pm(1)
            cncf(i,j,kgso4,k) = pm(2)
            cncf(i,j,kaso4,k) = pm(3)
            cncf(i,j,kh2o2,k) = amax1(pm(4),bdnl(kh2o2))
            cncf(i,j,ko3,k) = amax1(pm(5),bdnl(ko3))
            aso2conf(i,j,k)=so2av
c
c-----Determine nitrate equilibrium if this is a PM chemistry run, or if
c     its a Toxic chemistry run with nitrates
c
            if (lpmchm .or. (ltxchm .and. khno3.ne.0)) then
c
c               aso4 and gso4 should not be modified by remeql3
c               since prior versions of REMSAD have assumed
c               that all pariculate nitrate is nh4no3,
c               we will assume that here also. 
c               (tcm, 10/18/00)
                  tso4  = cncf(i,j,kaso4,k) + cncf(i,j,kgso4,k)
                  thno3 = cncf(i,j,khno3,k)
                  tnh3o = cncf(i,j,knh3,k)
                  pno3  = cncf(i,j,kpno3,k)
                  panh4 = cncf(i,j,knh4n,k) + cncf(i,j,knh4s,k)
                  call remeql3(tso4,thno3,tnh3o,rhcell(k),tcell(k),
     &                 pcell(k),pno3,panh4,paso4,i,j,k)
c
c     check against lower bound for these species
c
                  cncf(i,j,khno3,k) = amax1(thno3,bdnl(khno3))
                  cncf(i,j,knh3,k)  = amax1(tnh3o,bdnl(knh3))
                  cncf(i,j,kpno3,k) = amax1(pno3,bdnl(kpno3))
                  cncf(i,j,knh4n,k) = amax1(pno3,bdnl(knh4n))
                  cncf(i,j,knh4s,k) = amax1(panh4-pno3,bdnl(knh4s))
            endif
c
c     calculate soa partitioning
c
c           write(*,*) 'Calling soapart'
            aaa = etime(tarray)
            if (ksv1 .gt. 0) then
              svg(1) = cncf(i,j,ksv1,k)
              svg(2) = cncf(i,j,ksv2,k)
              svg(3) = cncf(i,j,ksv3,k)
              svg(4) = cncf(i,j,ksv4,k)
              svp(1) = cncf(i,j,ksoa1,k)
              svp(2) = cncf(i,j,ksoa2,k)
              svp(3) = cncf(i,j,ksoa3,k)
              svp(4) = cncf(i,j,ksoa4,k)
              mwv(1) = molwt(jind(ksv1))
              mwv(2) = molwt(jind(ksv2))
              mwv(3) = molwt(jind(ksv3))
              mwv(4) = molwt(jind(ksv4))
              call soapart(cncf(i,j,kpoc,k),rcon(koh),
     &                     pcell(k),tcell(k),deltat,svg,svp,
     &                     mwv,cpfac,nsv)
              cncf(i,j,ksv1,k) = svg(1)
              cncf(i,j,ksv2,k) = svg(2)
              cncf(i,j,ksv3,k) = svg(3)
              cncf(i,j,ksv4,k) = svg(4)
              cncf(i,j,ksoa1,k) = svp(1)
              cncf(i,j,ksoa2,k) = svp(2)
              cncf(i,j,ksoa3,k) = svp(3)
              cncf(i,j,ksoa4,k) = svp(4)
            endif
            bbb = etime(tarray)
            cpusoa = bbb - aaa + cpusoa
c           write(*,*) 'back from soapart'
c
c-----------------------------------------------------------------------
c-----Perform chemistry for toxics
c
            if (.not.ltxchm) goto 70
c
c-----Partition certain toxic species between vapor and particle phase
c     and load TOX array
c
            pm10 = 4.
            mfac = cpfac*(1013.25/pcell(k))*(tcell(k)/273.)
            do 60 l = ngas+1,nfspec
              if (l.eq.kpom1p .or. l.eq.kpom2p .or. l.eq.kdioxp .or.
     &            l.eq.katrazp) goto 60
              lsp = jind(l)
              pm10 = pm10 + molwt(lsp)*cncf(i,j,l,k)/mfac
  60        continue
c
            if (knaph.ne.0) tox(1) = cncf(i,j,knaph,k)
            if (kpom1.ne.0) then
              call toxpart(pm10,tcell(k),cncf(i,j,kpom1,k),
     &                     cncf(i,j,kpom1p,k),1)
              tox(2) = cncf(i,j,kpom1,k)
            endif
            if (kpom2.ne.0) then
              call toxpart(pm10,tcell(k),cncf(i,j,kpom2,k),
     &                     cncf(i,j,kpom2p,k),2)
              tox(3) = cncf(i,j,kpom2,k)
            endif
            if (kdiox.ne.0) then
              call toxpart(pm10,tcell(k),cncf(i,j,kdiox,k),
     &                     cncf(i,j,kdioxp,k),3)
              tox(4) = cncf(i,j,kdiox,k)
            endif
            if (katraz.ne.0) then
              call toxpart(pm10,tcell(k),cncf(i,j,katraz,k),
     &                     cncf(i,j,katrazp,k),4)
              tox(5) = cncf(i,j,katraz,k)
            endif
c
c-----Call TOX chemistry routine, then
c     repartition certain toxic species between vapor and particle phase
c
            call toxchm(dtf,tox,tcell(k))
c
            if (knaph.ne.0)
     &        cncf(i,j,knaph,k) = tox(1)
            if (kpom1.ne.0) then
              cncf(i,j,kpom1,k) = tox(2)
              call toxpart(pm10,tcell(k),cncf(i,j,kpom1,k),
     &                     cncf(i,j,kpom1p,k),1)
            endif
            if (kpom2.ne.0) then
              cncf(i,j,kpom2,k) = tox(3)
              call toxpart(pm10,tcell(k),cncf(i,j,kpom2,k),
     &                     cncf(i,j,kpom2p,k),2)
            endif
            if (kdiox.ne.0) then
              cncf(i,j,kdiox,k) = tox(4)
              call toxpart(pm10,tcell(k),cncf(i,j,kdiox,k),
     &                   cncf(i,j,kdioxp,k),3)
            endif
            if (katraz.ne.0) then
              cncf(i,j,katraz,k) = tox(5)
              call toxpart(pm10,tcell(k),cncf(i,j,katraz,k),
     &                     cncf(i,j,katrazp,k),4)
            endif
c
   70       continue
            do 75 l = 1,nfspec
              cncf(i,j,l,k) = amax1(cncf(i,j,l,k),bdnl(l))
   75       continue
   50     continue
c
          ndone = ndone + 1
          do k = 1, nlayer
            airfact = presf(i,j,k)/(1013.25*0.08206*temprf(i,j,k))
            airden = airfact*(1.0e-3)*(6.02e+23)
            fto3 = cncf(i,j,ko3,k)
            fth2o2 = cncf(i,j,kh2o2,k)
            ftoh = aradconf(koh,k)
            ftho2 = aradconf(kho2,k)
            ftno3 = aradconf(kno3,k)
            so2av = aso2conf(i,j,k)
            ftpec = cncf(i,j,kpec,k)
            icrs = (i-2)/nhf(ifine) + ixfb(ifine)
            jcrs = (j-2)/nhf(ifine) + jyfb(ifine)
            atcl20 = cl2con(icrs,jcrs)
            hmax = 2000.
            if (atcl20 .gt. 1.e-4) hmax = 1000.
            h2 = htf(i,j,k)
            if (k .gt. 1) then
              h1 = htf(i,j,k-1)
            else
              h1 = 0.
            endif
            call solar(ctlatf(i,j),-ctlonf(i,j),tz,nyr,nmth,nday,
     &                 time,zcoef)
            zenith = (180./pi)*acos(zcoef)
            if (zenith .lt. 85.) atcl20 = 0.0
            if (h1 .gt. hmax) then
              atcl2 = 0.
            else
              atcl22 = (hmax - amin1(hmax,h2))/hmax * atcl20
              atcl21 = (hmax - h1)/hmax * atcl20
              atcl2 = (amin1(hmax,h2)-h1)/(h2-h1)*.5*(atcl22+atcl21)
            endif
c
            lprnt = .false.
            if (mod(ndone,100000) .eq. 0) then
              if (mod(k,4) .eq. 1) lprnt = .true.
            endif
            do ihg = 1,mxhg
              if (khg0(ihg) .ne. 0) then
                call fhgchem(flwch2or(i,j,k),temprf(i,j,k),presf(i,j,k),
     &                     ihg,dtf,airden,i,j,k,ftoh,ftho2,ftno3,
     &                     fth2o2,fto3,so2av,atcl2,ftpec,
     &                     cncf(i,j,khg0(ihg),k),cncf(i,j,khg2(ihg),k),
     &                     cncf(i,j,khgp(ihg),k),lprnt)
              endif
            end do
            lprnt = .false.
          end do
c     
c-----Do cloud mixing/wet dep in a loop over species
c     Perform cloud aqueous chemical conversions while loading cloud
c     and non-cloud concentration profiles
c
          fracp = amax1(fconvf(i,j),fstrtf(i,j))
          do 40 l = 1,nfspec
            lsp = jind(l)
            do 41 k = 1,nzf
              cnocld(k,l) = cncf(i,j,l,k)
              cincld(k,l) = cncf(i,j,l,k)
              if (fracp.eq.0. .and. ccell(k).eq.0.) goto 41
              if (lwet) wi(k) = wratf(i,j,k)
c
              if (k.lt.kfrzf(i,j)) then
                frac1 = amax1((ccell(k) - fracp),0.)/
     &                        (1. - fracp + 1.e-10)
                frac2 = 0.
                if (fracp.ne.0.) then
                  if (k.ge.kcldbf(i,j) .and. k.le.kcldtf(i,j)) then
                    frac2 = 1.
                  else
                    frac2 = amin1(ccell(k),fracp)/fracp
                  endif
                endif
c
                if (l.eq.kgso4) then
                  if (kgso4.lt.kaso4)
     &              delgso4(k) = -fnucl*cncf(i,j,l,k)
                  cnocld(k,l) = cnocld(k,l) + frac1*delgso4(k)
                  cincld(k,l) = cincld(k,l) + frac2*delgso4(k)
                elseif (l.eq.kaso4) then
                  if (kgso4.gt.kaso4)
     &              delgso4(k) = -fnucl*cncf(i,j,kgso4,k)
                  cnocld(k,l) = cnocld(k,l) - frac1*delgso4(k)
                  cincld(k,l) = cincld(k,l) - frac2*delgso4(k)
                endif
              endif
   41       continue
c
c-----------------------------------------------------------------------
c-----If there is rain in this column, invoke the cloud sub-model
c
             do k = 1, nlayer
                acnocld(k)  = cnocld(k,l) 
                acincld(k)  = cincld(k,l) 
             end do
c
c
               if (l .eq. ko3) then
                  lpro3 = .false.
               else
                  lpro3 = .false.
               end if
               deltadp = 0.0
c
            if (lwet .and. rainf(i,j).ge.0.01) then
c
c-----Convective precipitation
c
            if (fconvf(i,j).gt.0.) then
	      if ( ltxchm .or. lpmchm) then
c                 write (*,*) 'convmix =',i,j,ifine,xmdf(i,j)
                    call convmix(lprnt,nzf,l,kcldbf(i,j),kcldtf(i,j),
     &                   kfrzf(i,j),dtf,delfx(j,ifine),delfy(ifine),
     &                   pstarf(i,j),xmdf(i,j),rainf(i,j),
     &                   sigmaf(1,ifine),zcell,tcell,
     &                   pcell,wi,acincld,deltadp,idebug,ierr,flqdh2o,i,j,lpro3)
                if (ierr.ne.0) then
                  write(*,*)'Stopping in RCHEMF at i,j',i,j
                  write(*,*)'Too many convective sub-steps'
                  stop
                endif
	      endif
c             if (ierr.ne.0) then
c               write(*,*)'Stopping in RCHEMF at i,j',i,j
c               write(*,*)'Too many convective sub-steps'
c               stop
c             endif

              deltadp = fconvf(i,j)*deltadp
c
c-----Stratiform precipitation
c
            elseif (fstrtf(i,j).gt.0.) then
                  call strtmix(lprnt,nzf,l,kcldbf(i,j),kcldtf(i,j),
     &                 kfrzf(i,j),dtf,delfx(j,ifine),delfy(ifine),
     &                 rainf(i,j),pstarf(i,j),sigmaf(1,ifine),
     &                 zcell,tcell,pcell,qcell,
     &                 wi,acincld,deltadp,idebug,flqdh2o,i,j,lpro3)
               deltadp = fstrtf(i,j)*deltadp
            else
              write(*,*)'ERROR (RCHEMF): '
              write(*,*)'Non-zero rain, but zero cloud fractions!'
              write(*,*)'i,j,rain,fconv,fstrt'
              write(*,*) i,j,rainf(i,j),fconvf(i,j),fstrtf(i,j)
              stop
            endif
            fdpflo(l,ifine) = fdpflo(l,ifine) - deltadp
            wpositf(i,j,l) = wpositf(i,j,l) + molwt(lsp)*deltadp/
     &                       (delfy(ifine)*delfx(j,ifine))
c
            endif
c
 2001       format (a,3(1x,i3),10(1x,e10.4))
            if (lpro3) then
               if (deltadp .ne. 0.0) then
                  write (81,2001) 'fine',i,j,ifine,deltadp,acincld(1),wpositf(i,j,l),fconvf(i,j),fstrtf(i,j)
               end if
            end if
c     
c-----Recombine in-cloud profiles with non-cloud profiles
c
             do k = 1, nlayer
                cnocld(k,l)  = acnocld(k) 
                cincld(k,l)  = acincld(k) 
             end do
c
            do 45 k = 1,nzf
              cncf(i,j,l,k) = fracp*cincld(k,l) + (1. - fracp)*cnocld(k,l)
  45        continue
  40      continue
c
c-----End loop on I and J
c
   99   continue
  100 continue
c
      return
      end
