c
c     Copyright 1996, 2001 Systems Applications International
c       The REMSAD modeling system software is the property of SAI
c       and is protected by registered copyrights. Distribution of
c       the software outside of your organization is not permitted.
c       Preparation and distribution of any derivative software using
c       any portions of the REMSAD code, without the express permission
c       of SAI, is also not permitted. By accepting the REMSAD code
c       provided in this release, you are also accepting the
c       limitations, terms, and provisions of use as noted above.
      subroutine scavrat(lprnt,nz,rain,clw,rlw,
     $                   lconv,kbot,ktop,kfreez,zi,wrat)
c
c-----ATDM v1.00
c
c     Calculates decay rates for wet scavenging of sulfate given cloud 
c     parameters for a model column.
c
c-----95/05/22:
c     C Emery SAI
c-----95/06/19:
c     -revised to be incorporated into ATDM (updated argument list, altered
c      units of wrat from 1/min to 1/s)
c-----95/08/17:
c     -removed fnucl from rate equations; to be applied at time of wet dep
c-----95/10/20:
c     -fixed some bugs and unforseen problems
c
c     Decay rates for other PM species are assumed to be constant fractions 
c     of the sulfate rates, based on the hygroscopic nature of each.
c     Methodology follows that of Scott (JAM, 1978: p. 1375-1389)
c
c     Note: the fraction of particles > 1 micron diameter (flrge) is 
c     species-dependent and is therefore applied to subcloud scavenging rates
c     in WETDEP.  
c
c     Input arguments:
c               lprnt:          diagnostic debug print flag
c               nz:             number of layers
c               rain:           rain rate (in/hr)
c               clw:            cloud liquid water mix ratio (g/g)
c               rlw:            rain liquid water mix ratio (g/g)
c               lconv:          convection flag
c               kbot:           layer index of cloud bottom
c               ktop:           layer index of cloud top
c               kfreez:         layer index of freezing level
c               zi:             profile of layer interface heights (m)
c     Output arguments:
c               wrat:           profile of washout rates (1/s)
c     Output arguments:
c     Files written:
c               6:              standard output file unit (*)         
c     Called by: 
c               clddiag   clddiagf 
c
      include 'parfin.cmd'
      real zi(nz), wrat(nz), vel(mxfz), xm(mxfz)
      real clw(nz), rlw(nz)
      real dz(mxfz)
      logical lconv,lprnt

      data c1 /5.2e-3/

      rainrat = 25.4*rain
      if (lprnt) write(*,*)'Rainfall rate: ',rainrat
      do 5 k = 1,nz
        dz(k) = zi(k)
        if (k.gt.1) dz(k) = zi(k) - zi(k-1)
        wrat(k) = 0.
    5 continue
c
c-----Set precipitation water content at top of riming zone (xm0), and find
c     effective rainfall rate (rain0), hydrometeor diameter (diam0), and
c     initial hydrometeor velocity (vel0)
c
      xm0 = 3.15e-3
      rain0 = (xm0/7.14e-2)**(1/0.88)
      diam0 = 8.95e-4*rain0**(0.21)
      vel0 = 130.*sqrt(diam0)
      if (lprnt) write(*,*)'xm0,rain0,diam0,vel0'
      if (lprnt) write(*,*) xm0,rain0,diam0,vel0
c
c-----Calculate precipitation water content (xmf), final diameter (diamb) 
c     and velocity of hydrometeor (velb) at cloud base as a function of 
c     rainfall rate
c
      diamb = 8.95e-4*rainrat**(0.21)
      velb = 130.*sqrt(diamb)
      xmf = 0.0714*rainrat**(0.88)
      if (lprnt) write(*,*)'diamb,velb,xmf'
      if (lprnt) write(*,*) diamb,velb,xmf
c         
c-----Estimate depth of riming zone and find average cloud water in riming 
c     zone (cwbar)
c     Assume the product C1xT = 4 for convective clouds;
c                             = 2 for stratiform clouds 
c
      if (lconv) then
        trim = 4./c1
      else
        trim = 2./c1
      endif
      zrim = trim*(velb + vel0)/2.
      zbot = 0.
      if (kbot.gt.1) zbot = zi(kbot-1)
      do 10 k = kbot,ktop
        zmid = zi(k)/2.
        if (k.gt.1) zmid = (zi(k) + zi(k-1))/2.
        zdif = zmid - zbot
        if (lprnt) write(*,*)'zdif: ',zdif
        if (zdif.gt.zrim) then
          krim = k - 1
          goto 11
        endif
   10 continue
      krim = ktop - 1
   11 if (krim.lt.kbot) krim = kbot
      zdif = zi(krim) - zbot
      trim = 2.*zdif/(velb + vel0)
      c1t = c1*trim
      cwbar = (3.12 + 0.88*alog(rainrat))/c1t
      if (lprnt) write(*,*)'krim,trim,zrim,zdif,c1t,cwbar'
      if (lprnt) write(*,*) krim,trim,zrim,zdif,c1t,cwbar
      if (lprnt) write(*,*)'rlw(krim), clw(krim)'
      if (lprnt) write(*,*) rlw(krim), clw(krim)
c
c-----Determine washout efficiency for layer just above riming zone (xi)
c     and washout rate (wrat) (this is a function of cloud type )
c         
      if (.not.lconv .and. krim+1.ge.kfreez) then
        wrat(krim+1) = 0.
      else
c       xi = xm0/cwbar
        if (rlw(krim+1) .lt. 1.e-08) rlw(krim+1) = 1.e-08
        if (clw(krim+1).le.1.0E-8) clw(krim+1) = 1.0E-8
        xi = rlw(krim+1)/clw(krim+1)
        wrat(krim+1) = xi/(435.*rainrat**(-0.71) + 1200.)
      endif
      if (lprnt) write(*,*) 'xm0, rainrat: ', xm0, rainrat
      if (lprnt) write(*,*)'KRIM+1: xi,wrat'
      if (lprnt) write(*,*) xi,wrat(krim+1)
c
c-----Loop from cloud base to top of riming zone to get profile of 
c     precipitation water content (xm(k)) and hydrometeor velocity (vel(k))
c
c     xm(kbot) = xmf
c     xm(krim+1) = xm0
c     vel(kbot) = velb
c     vel(krim+1) = vel0
c     if (lprnt) write(*,*)'k,vel,xm: ',kbot,vel(kbot),xm(kbot)
c     do 20 k = kbot,krim-1
c       dt = dz(k)/vel(k)
c       xm(k+1) = xm(k)*exp(-c1*dt*cwbar)
c       ran = (xm(k+1)/7.14e-2)**(1/0.88)
c       diam = 8.95e-4*ran**(0.21)
c       vel(k+1) = 130.*sqrt(diam)
c       velbar = (vel(k) + vel(k+1))/2.
c       dt = dz(k)/velbar
c       xm(k+1) = xm(k)*exp(-c1*dt*cwbar)
c       ran = (xm(k+1)/7.14e-2)**(1/0.88)
c       diam = 8.95e-4*ran**(0.21)
c       vel(k+1) = 130.*sqrt(diam)
c       if (lprnt) write(*,*)'k,vel,xm: ',k+1,vel(k+1),xm(k+1)
c  20 continue
c
      do 20 k = kbot,krim+1
        if (rlw(k) .lt. 1.e-08) rlw(k) = 1.e-08
        ran = (rlw(k)/7.14e-2)**(1/0.88)
        diam = 8.95e-4*ran**(0.21)
        vel(k) = 130.*sqrt(diam)
        if (lprnt) write(*,*)'k,vel,rlw: ',k,vel(k),rlw(k)
   20 continue
c
c-----Loop from top of riming zone to cloud base, and estimate washout
c     efficiency and washout rate in each layer
c
c     c1tsum = 0.
c     do 30 k = krim,kbot,-1
c       dt = 2.*dz(k)/(vel(k+1) + vel(k))
c       c1tsum = c1tsum + c1*dt
c       wrat(k) = (xm(k) - xm(k+1))/(cwbar*dt)
c       if (lprnt) write(*,*)'k,dz,dt,wrat(k)'
c       if (lprnt) write(*,*) k,dz(k),dt,wrat(k)
c  30 continue
c     if (lprnt) write(*,*)'c1t after integration: ',c1tsum
      c1tsum = 0.
      do 30 k = krim,kbot,-1
        dt = 0.5*(dz(k)/vel(k) + dz(k+1)/vel(k+1))
        c1tsum = c1tsum + c1*dt
        if (clw(k).le.1.0E-8) clw(k) = 1.0E-8
        wrat(k) = (rlw(k+1)-rlw(k))/(clw(k)*dt)
        if (wrat(k) .lt. 0.) wrat(k) = 0.
        if (lprnt) write(*,*)'k,dz,dt,wrat(k)'
        if (lprnt) write(*,*) k,dz(k),dt,wrat(k)
   30 continue
      if (lprnt) write(*,*)'c1t after integration: ',c1tsum
c
c-----Loop from just below cloud base to first layer to find washout efficiency
c     of large (>1 um) particles by impaction
c
      if (kbot.eq.1) goto 41
      do 40 k = kbot-1,1,-1
        wrat(k) = 2.14e-2*c1*rainrat**(0.88)
        if (lprnt) write(*,*)'k,wrat'
        if (lprnt) write(*,*) k,wrat(k) 
   40 continue
c
   41 return
      end
