c
c     Copyright 1996, 2001 Systems Applications International
c       The REMSAD modeling system software is the property of SAI
c       and is protected by registered copyrights. Distribution of
c       the software outside of your organization is not permitted.
c       Preparation and distribution of any derivative software using
c       any portions of the REMSAD code, without the express permission
c       of SAI, is also not permitted. By accepting the REMSAD code
c       provided in this release, you are also accepting the
c       limitations, terms, and provisions of use as noted above.
c
      subroutine wetdep(ldiag,nz,lind,ktop,kbot,kfreez,dx,dy,deltat,
     &        pstar,rain,sigma,zi,ti,pi,wrat,alfa,deltadp,lqdh2o,ix,jx,lpro3)
c
c-----ATDM v1.00
c
c     Adjusts concentrations for all species in a given
c     column due to precipitation scavenging
c
c-----95/04/18:
c     -Based on RTM-II and UAM-V code, restructured and improved for ATDM
c     -Henry's Law equilibrium constants and temperature dependence factors
c      moved to AQDAT.F
c-----95/06/23:
c     -revised mfac and vfac for use with sigma rather than height
c     -added sigma and pstarf
c     -added data statements for grav and amolwt
c-----95/10/18:
c     -Modified gas-equilibrium section for ppm rather than ug/m3, and
c      revised final (ground level) wet deposition flux calculation for 
c      proper units (moles)
c-----95/10/20:
c     -Added limitation that aqueous gas flux <= 0 in each layer
c-----95/10/25:
c     -revised gas scavenging flux, and fixed some errors
c-----95/12/12:
c     -modified code to treat nh4n as pno3 &
c      nh4s as aso4
c-----96/09/16:dfg
c     -added local array lmap for mapping species when Micro CB4 is
c      specified.
c     -added local variable lfirst.
c     -added calculations of ekh, ek1d, ek2d using arrays k0,dhor,
c      kaq1,tkaq1,kaq2,tkaq2 when Micro CB4 mechanism is used. 
c     -added included 'ctlflg.cmd' because lmech is used.
c-----97/8/13:dfg
c     -add some notes to the calculations.
c-----98/2/9:dfg
c     -fixed the error on species index for mapping when lmech=.true. 
c      This error did not affect the results when only PM option is specified.
c-----98/3/9:dfg
c     -added the skip of deposition calculation for ISOP.
c-----99/4/23:dfg
c     -changed the actual species number for CO, NTR and ISOP to kco, kntr,
c      kisop when skipping the calculation so that the number won't be changed
c      when different mechanism is specified.
c
c     Gaseous removal based on equilibrium scavenging assumption
c     (Hales and Sutter, 1979)
c     SO4 removal based on parameterization of Scott (1978); removal rates for
c     other PM species are set to fractions of SO4 rates using Fc
c     Current implementation: assume cloud pH = 4.5
c
c     Input arguments:
c               nz:             number of layers       
c               lind:           species index
c               ktop:           layer index of cloud top
c               kfreez:         layer index of freezing level
c               dx:             cell size in x-direction (km)
c               dy:             cell size in y-direction (km)
c               deltat:         timestep (s)
c               pstar:          column value of P* (mb)
c               rain:           rainfall rate (in/hr)
c               sigma:          profile of sigma structure
c               zi:             profile of layer interface heights (m)
c               ti:             profile of temperature (K)
c               pi:             profile of pressure (mb)
c               wrat:           wet scavenging rate for sulfate (1/s)
c               alfa:           profile of concentrations (ppm)
c     Output arguments:
c               alfa:           profile of concentrations (ppm)
c               deltadp:        deposited mass (moles)
c     Called by:
c               convmix   strtmix
c
      include 'parm.cmd'
      include 'parchm.cmd'
      include 'comchem.cmd'
      include 'aerosl.cmd'
      include 'ctlflg.cmd'
c
      dimension sigma(nz+1),zi(nz),ti(nz),pi(nz),wrat(nz),alfa(nz)
      real mfac,lwc
      real*4 lqdh2o(nz)
      logical ldiag,lpro3
      logical lfirst
      data ph /4.5/, grav /9.8/, amolwt /28.8/
c 
c-----Initilize some constants
c
      if (lpro3) then
      write (84,*) 'wetdep',ix,jx,lpro3
      end if
c
      ll = jind(lind)
      hplus  = 10.0**(-ph)
      hplus2 = hplus*hplus
c
c     Convert rain (ih/h) to rainmm (mm/h)
c
      rainmm = 25.4*rain
c
c     Convert rain (ih/h) to rainm (m/h)
c
      rainm  = rain/39.37
c
c     Calculate the median raindrop diameter (m) and raindrop velocity (m/s)
c
      diam   = 8.95e-4*rainmm**(0.21)
      rainvel = 130.*diam**(0.5)
c
      mfac = pstar/(10.*grav*amolwt)
      deltadp = 0.
c
c     write(*,*) 'lind, ngas, ll, mxgas: '
c     write(*,*) lind, ngas, ll, mxgas
      if (lind.gt.ngas) goto 100
c
c---- Check if it is run with mechanism specified, the species order
c     will be different.
c
      lkk=kind(ll)
      if (lmech) lk=lmap(lkk)
c      write (*,*) 'll, kind(ll), jind(lind), lind, lkk,lk: '
c      write (*,*) ll, kind(ll), jind(lind), lind, lkk,lk
c
c   Skip the calculation for CO(8), NTR(12), & ISOP(15).
c
      if (lmech) then
        if (lkk.eq.kco   .or. lkk.eq.kntr .or. lkk.eq.kisop .or.
     &      lkk.eq.kterp) then
c          do k=ktop,1,-1
c            alfa(k)=0.
c          end do
          return
        endif
      endif
c
c
c-----Calculate wet deposition for gasses within layers in which temperature
c     is greater than 273 K
c
c     write (*,*) 'before 5 in wetdep'
      itop = 0
      do 5 k = ktop,1,-1
        dsigma = sigma(k) - sigma(k+1)
        if (k.ge.kfreez) goto 5
        delh = zi(k)
        if (k.gt.1) delh = zi(k) - zi(k-1)
        vfac = 1.e6*dx*dy*dsigma
c
c       Calculating the liquid water content (l water/l air) and convert
c       it to (atm l/mole).  i.e. multiply by RT, R=0.08206 (l atm/mol K)
c       (0.08206/3600. = 2.279E-05)
c
         zlwc = 2.278e-5*ti(k)*rainm/rainvel
c
c now lwc is taken from rain and cloud water mixing ratios
c
        lwc = lqdh2o(k)
        if (lpro3) then
           write (84,*) 'wetdep-1', ix,jx,lwc, zlwc, k
        end if
        if (lwc .eq. 0.0) go to 5
        if (lpro3) then
           write (84,*) 'wetdep-2', ix,jx,lwc, zlwc, k
        end if
c     
c       Calculating the numerator for scavenging rate (1/min), 
c       i.e. RT*rainm
c       (0.08206/60. = 1.367E-03)
c
        top = 1.367e-3*ti(k)*rainm
        tfac = 1./298. - 1./ti(k)
c
        if (lmech) then
c           write (*,*) 'lmech is true'
c        write(*,*) 'wetdep-ll,lkk,lmap= ',ll,lkk,lk
c***DFG
          ekh = k0(lk) * exp(dhor(lk)*tfac)
          ek1d = kaq1(lk) * exp(tkaq1(lk)*tfac)
          ek2d = kaq2(lk) * exp(tkaq2(lk)*tfac)
        else
c           write (*,*) 'lmech is false'
          ekh  = ekh0(ll)*exp(tdep(ll)*tfac)
          ek1d = ek1d0(ll)*exp(tdep1d(ll)*tfac)
          ek2d = ek2d0(ll)*exp(tdep2d(ll)*tfac)
        endif
c
c        write (*,*) k0(lk),kaq1(lk),kaq2(lk),ekh0(ll),ek1d0(ll),ek2d0(ll)
c        write (*,*) dhor(lk),tkaq1(lk),tkaq2(lk),tdep(ll),tdep1d(ll),tdep2d(ll)
c        write (*,*) ekh,ek1d,ek2d, tfac
c
        sol = 1.0/(ekh + ekh*ek1d/hplus + ekh*ek1d*ek2d/hplus2)
c        write (*,*) 'sol=', sol
        if (ldiag) write(*,*)'k,sol,top,lwc,alfa',k,sol,top,lwc,alfa(k)
c
        if (itop.eq.0) then
c           write (*, *) 'top=', itop
          itop = 1
          rdecay = -top/(60.*delh*(lwc + sol))
          delc = alfa(k)*(exp(rdecay*deltat) - 1.)
          deltadp = -delc*mfac*vfac
          if (lpro3) then
             if (ix .eq. 18 .and. jx .eq. 86) then
                write (82,*) 'first',ix,jx,k,delc,mfac,vfac,deltadp
             end if
          end if
          flux = delc*mfac*dsigma/deltat
          alfa(k) = alfa(k) + delc
c         if (delc .gt. 0.) then
c           write(*,*) 'rdecay,top,delh,lwc,zlwc,sol: '
c           write(*,*) rdecay,top,delh,lwc,zlwc,sol
c           write(*,*) 'delc, alfa(k),deltat: '
c           write(*,*) delc, alfa(k),deltat
c           write(*,*) 'deltadp,mfac,vfac: '
c           write(*,*) deltadp,mfac,vfac
c           write(*,*) 'flux,dsigma: '
c           write(*,*) flux,dsigma
c         endif
          if (ldiag) write(*,*)'alfa,flux,deltadp',
     &                          alfa(k),flux,deltadp
          goto 5
        else
c         a = -flux/(mfac*dsigma)
          a = flux/(mfac*dsigma)
          b = -top/(60.*delh*(lwc + sol))
          dc1 = alfa(k)*(exp(b*deltat) - 1.)
          aoverb = a/b
          delmol = aoverb*mfac*vfac
c           write(*,*) 'aoverb, delmol, deltadp: '
c           write(*,*) aoverb, delmol, deltadp
          if (delmol.gt.deltadp) then
            delmol = deltadp
            aoverb = delmol/(mfac*vfac)
          endif
          dc2 = aoverb*(exp(b*deltat))
          deltadp = deltadp - dc1*mfac*vfac - dc2*mfac*vfac
          delc = dc1 + dc2
c            write(*,*) 'deltadp, delc, dc1, dc2: '
c            write(*,*) deltadp, delc, dc1, dc2
          if (lpro3) then
             if (ix .eq. 18 .and. jx .eq. 86) then
                write (82,*) 'second',ix,jx,k,delc,mfac,vfac,deltadp
             end if
          end if
          flux = -deltadp*dsigma/vfac/deltat
c         delc = (alfa(k) + a/b)*(exp(b*deltat) - 1.)
c         delmol = delc*mfac*vfac
c           write(*,*) 'delc, deltadp, flux: '
c           write(*,*) delc, deltadp, flux
c         if (delmol.gt.deltadp) delc = deltadp/mfac/vfac
c         deltadp = deltadp - delc*mfac*vfac
c         if (lpro3) then
c            if (ix .eq. 18 .and. jx .eq. 86) then
c               write (82,*) 'second',ix,jx,k,delc,mfac,vfac,deltadp
c            end if
c         end if
c         flux = flux + delc*mfac*dsigma/deltat
          alfa(k) = alfa(k) + delc
c         if (delc .gt. 0.) then
c           write(*,*) 'a,flux,mfac,dsigma: '
c           write(*,*) a,flux,mfac,dsigma
c           write(*,*) 'b,top,delh,lwc,zlwc,sol: '
c           write(*,*) b,top,delh,lwc,zlwc,sol
c           write(*,*) 'delc,alfa(k),deltat: '
c           write(*,*) delc,alfa(k),deltat
c           write(*,*) 'delmol,vfac: '
c           write(*,*) delmol,vfac
c           write(*,*) 'deltadp: '
c           write(*,*) deltadp
c         endif
          if (ldiag) write(*,*)'alfa,flux,deltadp',
     &                          alfa(k),flux,deltadp
        endif
    5 continue
c     write (*,*) 'after 5 in wetdep'
      return
c
c-----Calculate wet deposition for particulates
c
  100 continue
      do 20 k = ktop,1,-1
        dsigma = sigma(k) - sigma(k+1)
        vfac = 1.e6*dx*dy*dsigma
        if (k.ge.kbot) then
          frac = exp(-fc(lind)*fnucl*wrat(k)*deltat) - 1.
          frac = amax1(frac,-fc(lind)*fnucl)
        else
	  if ( lind .eq. knh4n ) then
	     rflrg=flrge(iszdist(kpno3))
	  elseif ( lind .eq. knh4s ) then
	     rflrg=flrge(iszdist(kaso4))
          else
	     rflrg=flrge(iszdist(lind))
          endif
          frac = exp(-rflrg*wrat(k)*deltat) - 1.
          frac = amax1(frac,-rflrg)
        endif
        delc = frac*alfa(k)
        deltadp = deltadp - delc*vfac*mfac
c       if (delc .gt. 0.) then
c         write(*,*) 'k, delc,frac,alfa(k): '
c         write(*,*) k, delc,frac,alfa(k)
c         write(*,*) 'deltadp,vfac,mfac: '
c         write(*,*) deltadp,vfac,mfac
c         write(*,*) 'lind, fc(lind),fnucl,wrat(k),deltat: '
c         write(*,*) lind, fc(lind),fnucl,wrat(k),deltat
c         write(*,*) 'rflrg,flrge(iszdist(lind)),iszdist(lind): '
c         write(*,*) rflrg,flrge(iszdist(lind)),iszdist(lind)
c       endif
        if (lpro3) then
           if (ix .eq. 18 .and. jx .eq. 86) then
              write (82,*) 'third',ix,jx,k,delc,mfac,vfac,deltadp
           end if
        end if
        alfa(k) = alfa(k) + delc
        if (ldiag) write(*,*)'k,frac,alfa,deltadp',
     &                        k,frac,alfa(k),deltadp
   20 continue
c      write (*,*) 'after 20 in wetdep'
      return
c
      end
